# QIS installation guide

To follow this guide you will need to have some knowledge of Linux server administration,
and be familiar with using the Linux command line interface and common utilities.
If this isn't you, by all means read on, but don't be surprised if the process quickly
feels rather involved.

Apart from itself, QIS incorporates 3 major component applications:

* Apache (with Python and mod_wsgi)
* PostgreSQL
* Memcached

Each of these can be run on a single server or across multiple servers, and each can be
run as one application instance or many (clustered or load balanced).
To simplify this guide, one instance of each application will be installed on one server.

QIS requires a Linux-based operating system. However, development and testing takes place
on Mac OS X, so it is likely that other unix-based operating systems will also work.
For a list of operating systems that are known to work, see the
[operating systems](operating_systems.md) notes.

The examples given here assume a Fedora-based system (with the `systemd` init system and
the `yum` installer). For Debian-based systems, replace `yum` with `apt-get`, and be aware
that the commands, package names and file installation locations may differ slightly.

### Important! About the QIS application files

This guide will later require that you have 2 QIS package files:

* `Quru Image Server-2.xx.tar.gz` - the main QIS Python web application
* `dependencies.tar.gz` - the application's Python dependencies,
  including compiled C extensions as platform-specific binaries

These can be generated by running `make` from the QIS source code, but be aware that
one of the mandatory dependencies - `qismagick` - is not currently open source
(this is due to some licencing restrictions which we are working on resolving)
and will not be present by default. **QIS will not run without this file!**

To obtain a copy of `qismagick` for your platform, please email a request to info@quru.com

## Update the operating system

First it is recommended to update the operating system as a whole, to bring in the latest
software updates and security fixes.

	$ sudo yum -y update

## Running in the cloud?

If you will be scripting your installation to run in The Cloud, for example at
Amazon Web Services to provide elastic scaling, you will likely need the `cloud-init`
package installed.

	$ sudo yum -y install cloud-init

## Install Memcached

Install the package:

	$ sudo yum -y install memcached

Raise the memory limit from its default 64MB to a more useful value.
Edit `/etc/sysconfig/memcached` and change `CACHESIZE`:

	CACHESIZE="1024"

When the Memcached service is full, your server must still have enough RAM free to
run the other applications (if relevant) and cache operating system files, without
swapping. This is an advanced topic, see the [tuning guide](tuning.md) for more information.

By default, Memcached accepts remote connections from anywhere*. If Apache will be
installed on a different server, this is fine, but add a firewall rule to prevent
connections from elsewhere. If Apache will be installed on the same server, tell
Memcached to accept local connections only. Edit `/etc/sysconfig/memcached` and
change `OPTIONS`:

    OPTIONS="-l 127.0.0.1"

  *Some Debian-based systems set `-l 127.0.0.1` by default in `/etc/memcached.conf`.

Set the service to start on boot and start it now:

	$ sudo systemctl enable memcached
	$ sudo systemctl start memcached

## Install PostgreSQL

Set your language and character set (where `UTF8` is preferred), as these are used by
Postgres installer when determining the default language support:

	$ export LANG=en_GB.utf8
	$ export LC_ALL=en_GB.utf8

Install the package:

	$ sudo yum -y install postgresql-server

Initialise a new Postgres database collection:

	$ sudo -u postgres initdb -D /var/lib/pgsql/data --auth-host=trust --auth-local=trust

Set the Postgres tuning options in `/var/lib/pgsql/data/postgresql.conf`.
This is an advanced topic, see the tuning guide and the
[Postgres documentation](https://wiki.postgresql.org/wiki/Tuning_Your_PostgreSQL_Server)
for more information. Typically though the following settings are updated:

* `shared_buffers` - raise to the amount of RAM you want to dedicate to Postgres (e.g. `512MB`)
* `effective_cache_size` - the value of `shared_buffers` plus however much server RAM you expect
  to be free after all other applications are running
* `checkpoint_segments` - set to `16`
* `checkpoint_completion_target` - set to `0.9`

If Apache will be installed on a different server, enable TCP/IP connections to the Postgres
service. This has security implications, and the example below allows remote connections from
anywhere. Consult the [Postgres documentation](http://www.postgresql.org/docs/9.2/static/auth-pg-hba-conf.html)
for more information. You can skip this if Apache will be installed on the same server.

	$ sudo echo "listen_addresses = '*'" >> /var/lib/pgsql/data/postgresql.conf
	$ sudo echo "host    all    all    0.0.0.0/0    md5" >> /var/lib/pgsql/data/pg_hba.conf

Set the service to start on boot and start it now:

	$ sudo systemctl enable postgresql
	$ sudo systemctl start postgresql

If the service fails to start, see the _raising the system limits_ section below.
But assuming success, create a QIS database user and new empty database:

	$ sudo -u postgres psql -c "CREATE ROLE qis WITH LOGIN PASSWORD 'password'"
	$ sudo -u postgres createdb --owner qis qis-cache
	$ sudo -u postgres createdb --owner qis qis-mgmt

The `qis` user password should be set to a secure value.
The password is not used though if Apache is installed on the same server.

## Add an operating system user to run QIS

The QIS web application runs under its own local user account. This allows the
application's processes, file and directory permissions to be subject to standard
access controls.

If you require any other web-based systems to integrate with QIS, it can be useful
to make the `qis` user a member of the `apache` group. In this case you will first
need to install the Apache package for the `apache` group to be created.

	$ sudo yum -y install httpd
	$ sudo useradd --groups apache --comment "Quru Image Server" --home /opt/qis --system --shell /sbin/nologin qis

## Install QIS

Install the required packages and useful utilities:

	$ sudo yum -y install ImageMagick python postgresql ghostscript \
	                      openssl openldap libmemcached
	$ sudo yum -y install curl wget openssh-clients pwgen tar zip unzip python-pip

You will next require the 2 QIS distribution files:
`Quru Image Server-2.xx.tar.gz` and `dependencies.tar.gz`.

Note that the dependencies file is specific to server architecture and Python version,
and you require the correct version for your operating system. The following commands
assume these files will be in your home directory.

Create the QIS base directory and install the files:

	$ sudo mkdir /opt/qis && sudo chown qis:qis /opt/qis
	$ cd /opt/qis
	$ sudo -u qis mkdir -p lib/python2.7
	$ cd lib/python2.7
	$ sudo -u qis tar -xvf ~/dependencies.tar.gz
	$ cd ../..
	$ sudo -u qis tar --strip-components=1 -xvf ~/Quru\ Image\ Server-2.xx.tar.gz

The installation should look like this:

	$ ls -l /opt/qis/
	total 52
	drwxr-xr-x 2 qis apache 4096 May  22 10:27 conf
	drwxr-xr-x 2 qis apache 4096 May  22 10:27 deploy
	drwxr-xr-x 2 qis apache 4096 May  22 10:27 doc
	drwxr-xr-x 2 qis apache 4096 May  22 10:27 icc
	drwxr-xr-x 3 qis apache 4096 May  22 10:27 images
	drwxr-xr-x 3 qis apache 4096 May  22 10:27 lib
	drwxr-xr-x 2 qis apache 4096 May  22 10:27 licences
	drwxr-xr-x 2 qis apache 4096 May  22 10:27 logs
	-rw-r--r-- 1 qis apache  311 May  22 10:27 PKG-INFO
	-rw-r--r-- 1 qis apache  522 May  22 10:27 setup.cfg
	-rw-r--r-- 1 qis apache 1077 May  22 10:27 setup.py	
	drwxr-xr-x 5 qis apache 4096 May  22 10:27 src

## Install Apache and friends

Install the packages:

	$ sudo yum -y install httpd mod_ssl mod_wsgi logrotate

Set the language and character set (where `UTF8` is preferred) for the Apache
process by adding a few lines to the file `/etc/sysconfig/httpd`:

	# These entries resolve Python error "ascii codec can't decode byte"
	# when dealing with files and directories that contain non-ascii characters
	LANG=en_GB.UTF-8
	LC_ALL=en_GB.UTF-8

Create Apache configuration files to run QIS:

	$ cd /opt/qis
	$ sudo cp deploy/centos7/wsgi.conf /etc/httpd/conf.d/wsgi.conf
	$ sudo cp deploy/centos7/httpd.conf.sample /etc/httpd/conf.d/qis.conf
	$ sudo cp deploy/centos7/httpd-ssl.conf.sample /etc/httpd/conf.d/qis-ssl.conf

Substituting `centos7` for the directory name in `/opt/qis/deploy` that most closesly
matches your operating system. Ubuntu 14 and CentOS 7 ship with Apache v2.4, while
CentOS 6 ships with Apache v2.2. The two Apache versions have rather different
configuration files.

Tailor the `qis.conf` and `qis-ssl.conf` files for your server.
The following entries will need to be changed:

* `ServerName` - set this to be the host name (including domain name) of your server
* `WSGIDaemonProcess` - set the total number of processes (i.e. in both `conf` files)
  to be no more than the number of CPU cores in your server minus 1, minus 1 if
  Postgres is also installed here, minus 1 if Memcached is also installed here.
  If in doubt, start small and leave the default value of `2`.

If you understand Apache configuration, review that the other default settings meet
your needs. Then check the validity of the new `conf` files:

	$ httpd -t
	Syntax OK

You may also need to adjust the default Apache settings in `/etc/httpd/conf/httpd.conf`,
e.g. to raise the maximum number of workers. This is an advanced topic, see the tuning guide
and the [Apache documentation](http://httpd.apache.org/docs/current/misc/perf-tuning.html)
(especially the MPM settings) for more information. A guideline for the number of Apache
workers is 4 * `WSGIDaemonProcess` processes * `WSGIDaemonProcess` threads. By default,
Apache has worker limit of around 250.

Set the Apache service to start on boot. We'll start the service in this session after
performing the QIS configuration that follows below.

	$ sudo systemctl enable httpd

### Optional - HTTPS configuration

The `qis-ssl.conf` file enables HTTPS in Apache using the server's default certificate
files. This arrangement does work, but will give nasty security warnings in most web
browsers, and will require you to "add a security exception" or similar.

To avoid these security warnings, you will need certificate files that are specific
to the host name (domain name) of your server. You can buy SSL certificates from companies
such as [Thawte](https://www.thawte.com/), or obtain them free from
[StartSSL](https://www.startssl.com/) or [Let's Encrypt](https://letsencrypt.org/).

Installation instructions are provided by the certificate suppliers, but should only
involve copying the files onto your server and setting the locations to them in the
file `/etc/httpd/conf.d/qis-ssl.conf`.

### Optional - Set the cross-domain policy

Web browsers allow images, JavaScript and CSS files to be loaded from any domain.
Background data requests however are only allowed to be made to the same domain as
the originating web page by default. The zooming image viewer, slideshow/carousel
and gallery viewers all make background data requests. Therefore to enable the use
of these viewers (and the required public data APIs) on any web site, QIS sets the
following HTTP header in `qis.conf` and `qis-ssl.conf`:

    Header set Access-Control-Allow-Origin "*"

If you want to lock down this data access to a single domain, change the `*` value as
[described here](https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Access-Control-Allow-Origin).

### Optional - Logrotate configuration

The default Apache configuration (in `/etc/httpd/conf.d/qis.conf` and
`/etc/httpd/conf.d/qis-ssl.conf`) logs every image request to an Apache _access log_ file
in the `/var/log/httpd/` directory. The _logrotate_ utility (we installed above) is then
responsible for closing each log file and starting a new one. By default it does this once
per week, which is fine for a small to medium traffic web site, but a very busy site might
accumulate 1GB of logs per day.

If you have a very busy image server, you can instead swap the log files on a daily basis,
and also compress the archived logs to save disk space.

View the current logrotate configuration for Apache with:

	$ cat /etc/logrotate.d/httpd
	/var/log/httpd/*log {
	    missingok
	    notifempty
	    sharedscripts
	    delaycompress
	    postrotate
	        /bin/systemctl reload httpd.service > /dev/null 2>/dev/null || true
	    endscript
	}

To enable log compression, archive daily, and keep 7 days worth of logs, add the
`compress`, `daily`, and `rotate` directives so that the configuration becomes:

	/var/log/httpd/*log {
	    missingok
	    notifempty
	    sharedscripts
	    compress
	    delaycompress
	    daily
	    rotate 7
	    postrotate
	        /bin/systemctl reload httpd.service > /dev/null 2>/dev/null || true
	    endscript
	}

## Optional - Raising the system limits

Some Linux-based systems default to a low limit on the number of running processes
(including threads) and files (including network connections), which can cause
errors on a busy web server. To check these limits, run:

	$ ulimit -n -u
	open files                      (-n) 1024
	max user processes              (-u) 1024

### With systemd

On recent systems such as CentOS 7 that use systemd, system limits are controlled
at the service level. Do not edit the main service file, instead install a
service override file to raise the limits:

    $ cd /opt/qis
    $ sudo mkdir /etc/systemd/system/httpd.service.d
    $ sudo cp deploy/centos7/httpd-limits.conf /etc/systemd/system/httpd.service.d/limits.conf
    $ sudo systemctl daemon-reload
    $ sudo systemctl restart httpd.service

### Without systemd

On older systems such as CentOS 6, system limits are defined at the user level.
To raise the limits, install a configuration file into `/etc/security/limits.d/`:

    $ cd /opt/qis
    $ sudo cp deploy/centos6/limits.conf /etc/security/limits.d/qis.conf
    $ sudo service httpd restart

### Postgres

PostgreSQL, especially versions 9.2 and below, allocates a block of memory on startup that
may exceed the system's "shared memory" limit. If the Postgres service fails to start, check
the log file at `/var/lib/pgsql/data/pg_log/postgresql.log`. If there is an error regarding
shared memory, you will need to raise the shared memory limits.

To raise the shared memory limit to 16GB, run commands:

	$ sysctl -w kernel.shmmax=17179869184
	$ sysctl -w kernel.shmall=4194304

Then change (or add) the same values in `/etc/sysctl.conf`.
You can also refer to the [Postgres documentation](http://www.postgresql.org/docs/9.2/static/kernel-resources.html).

## Configuring the firewall

If your system has a firewall enabled, you will need to allow web traffic to
reach the server. How to do this is operating system specific. With `firewalld`
(Fedora 15+, CentOS 7+), run:

    $ sudo firewall-cmd --zone=public --permanent --add-service=http
    $ sudo firewall-cmd --zone=public --permanent --add-service=https
    $ sudo firewall-cmd --reload

With `ufw` (Ubuntu), run:

    $ sudo ufw allow http
    $ sudo ufw allow https

## Configuring Security Enhanced Linux (SELinux)

If your operating system has SELinux enabled, you must define some additional
disk permissions and install a policy file to allow the software components to
communicate with each other.

You can check this with `sestatus`:

	$ sestatus
	SELinux status:                 enabled
	SELinuxfs mount:                /sys/fs/selinux
	SELinux root directory:         /etc/selinux
	Loaded policy name:             targeted
	Current mode:                   enforcing
	Mode from config file:          enforcing
	Policy MLS status:              enabled
	Policy deny_unknown status:     allowed
	Max kernel policy version:      29

If the SELinux status is `disabled` or the current mode is not `enforcing`,
you can skip the rest of this section.

Otherwise, see if the `semanage` command is available, and if not install it:
 
	$ sudo yum install -y policycoreutils-python

Install the QIS security policy, and allow the Apache service to write to the
QIS logs and images directories:

	$ sudo semodule --install=/opt/qis/deploy/centos7/qis.pp
	$ sudo semanage fcontext --add --type httpd_log_t "/opt/qis/logs(/.*)?"
	$ sudo semanage fcontext --add --type httpd_user_rw_content_t "/opt/qis/images(/.*)?"
	$ sudo restorecon -rv /opt/qis/logs/
    $ sudo restorecon -rv /opt/qis/images/
    $ sudo restorecon -rv /opt/qis/conf/

If you are using NFS (network file system) to access your image directories,
you will also need to run:

	$ sudo setsebool -P httpd_use_nfs on

If you installed a TLS/SSL certificate for HTTPS support, set the security
context of the new files (set the directory names here for where you installed
the `crt/pem/key` files):

    $ sudo restorecon -rv /etc/ssl/certs/
    $ sudo restorecon -rv /etc/pki/tls/

If you will be taking your QIS user accounts from LDAP you will need:

	$ sudo setsebool -P httpd_can_connect_ldap on

## Configuring QIS

QIS has a large number of configuration settings, but you only need to set a few to get going.
One of these is the `SECRET_KEY` value, which as the name suggests, should be a value unique
to your site and be kept a secret. You can generate a secret key with the `pwgen` utility:

	$ pwgen -s 32 1
	zOWp8lBL1IMVEl9uWzPn2PD2ddtZTPRv

Create a new text file `/opt/qis/conf/local_settings.py` and add these lines:

	PUBLIC_HOST_NAME = "images.example.com"
	SECRET_KEY = "zOWp8lBL1IMVEl9uWzPn2PD2ddtZTPRv"

Where the host name is the same value you used when setting up Apache, and the secret key
the value you just generated. If you want to run multiple web servers (e.g. behind a load
balancer) then these settings need to be the same on every instance.

If you will be using a load balancer, or running your web server behind a reverse proxy,
you need to define how many hops there are (which is usually just `1`):

	PROXY_SERVERS = 1

If Memcached and Postres are installed on the same server, you can now skip to the
_first run_ section.

Or if Memcached is on a different server, you'll need to override the default value for the
`MEMCACHED_SERVERS` setting. To the `/opt/qis/conf/local_settings.py` file, add:

	MEMCACHED_SERVERS = ["mc.example.com:11211"]

Where `mc.example.com` is the local host name or IP address of your Memcached server,
and `11211` is the Memcached port. For multiple Memcached servers, use a comma separated
list such as `["11.22.33.44:11211", "11.22.33.45:11211"]`.

If Postgres is on a different server, you'll need to override the default values for the
2 `DATABASE_CONNECTION` settings. To the `/opt/qis/conf/local_settings.py` file, add:

	CACHE_DATABASE_CONNECTION = "postgresql+psycopg2://qis:password@db.example.com:5432/qis-cache"
	MGMT_DATABASE_CONNECTION  = "postgresql+psycopg2://qis:password@db.example.com:5432/qis-mgmt"

Where `qis:password` is the database username and password (as set in the _install PostgreSQL_
section), `db.example.com` is the local host name or IP address of your Postgres server,
and `5432` is the Postgres port.

To review the other default settings, view the contents of the file
`/opt/qis/src/imageserver/conf/base_settings.py`. You can override any of these settings
in your `local_settings.py` file. Do not edit `base_settings.py` directly because this
file will be overwritten the next time you upgrade QIS.

If you are using SELinux, set the security context of the new settings file:

    $ sudo restorecon -rv /opt/qis/conf/

## First run

The Memcached and Postgres services must already be running, then to start QIS,
run the Apache service:

	$ sudo systemctl start httpd

and check the QIS log file to see what happened:

	$ cat /opt/qis/logs/qis.log
	2015-05-22 10:37:54,054 qis_18     INFO     Quru Image Server v2.0.0-dev.4 engine startup
	2015-05-22 10:37:54,054 qis_18     INFO     Using settings base_settings + local_settings.py
	2015-05-22 10:37:54,058 qis_18     INFO     Cache usage currently 0 out of 1048576000 bytes (0%), holding 0 objects.
	2015-05-22 10:37:54,054 qis_19     INFO     Quru Image Server v2.0.0-dev.4 engine startup
	2015-05-22 10:37:54,055 qis_19     INFO     Using settings base_settings + local_settings.py
	2015-05-22 10:37:54,071 qis_19     INFO     Cache usage currently 0 out of 1048576000 bytes (0%), holding 0 objects.
	2015-05-22 10:37:54,309 qis_19     INFO     Cache control database created.
	2015-05-22 10:37:54,314 qis_19     INFO     Cache control database opened.
	2015-05-22 10:37:56,221 qis_19     INFO     Created new group 'Public'
	2015-05-22 10:37:56,222 qis_19     INFO     Created Public group
	2015-05-22 10:37:56,232 qis_19     INFO     Created new group 'Normal users'
	2015-05-22 10:37:56,233 qis_19     INFO     Created Normal users group
	2015-05-22 10:37:56,243 qis_19     INFO     Created new group 'Administrators'
	2015-05-22 10:37:56,244 qis_19     INFO     Created Administrators group
	2015-05-22 10:37:56,266 qis_19     INFO     Created new user account for username 'admin' with ID 1
	2015-05-22 10:37:56,267 qis_19     INFO     Created default admin user with password SHhUxIykPH
	2015-05-22 10:37:56,278 qis_19     INFO     Created new folder for path: /
	2015-05-22 10:37:56,278 qis_19     INFO     Created root folder
	2015-05-22 10:37:56,353 qis_19     INFO     Created default folder permissions
	2015-05-22 10:37:56,364 qis_19     INFO     Management + stats database opened
	2015-05-22 10:37:56,365 qis_19     INFO     Housekeeping task scheduler started
	2015-05-22 10:37:56,388 qis_19     INFO     Loaded imaging library: ImageMagick version: 688, Ghostscript delegate: 9.14
	2015-05-22 10:37:57,514 qis_18     INFO     Cache control database opened.
	2015-05-22 10:37:57,562 qis_18     INFO     Management + stats database opened
	2015-05-22 10:37:57,562 qis_18     INFO     Housekeeping task scheduler started
	2015-05-22 10:37:57,573 qis_18     INFO     Loaded imaging library: ImageMagick version: 688, Ghostscript delegate: 9.14

A user account for the `admin` user is created the first time you start QIS.
Note down the admin user's password, so `SHhUxIykPH` in this example.

You can see 2 sets of log entries here, one for process ID `18` and one for process `19`.
Each of these represents one `WSGIDaemonProcess` process (see the _install Apache_ section).
With the default configuration of 2 processes serving http and 2 serving https, you would
therefore expect to see 4 processes starting here and 4 sets of log entries.

Next, bring up the web interface and change the password for the admin user.

* In a web browser, navigate to `https://images.example.com/login/`
  (replacing the example host name with your own)
* Enter `admin` for the username
* Enter the password you noted from the log above
* Click _Sign in_
* Once logged in, hover over the _Administrator_ name in the top right corner, and choose _Edit account_
* In the _Password_ field, enter a new password
* Click _Apply_

By default, both public and logged in users have permission to view all images,
but not to upload or change them. If you want to review or change this:

* Sign in as above
* Select the _Administration_ option in the top right menu, and choose _Folder permissions_
* Choose a folder and group combination, e.g. "root folder" and "Public"
  * The root folder permissions are inherited by all other folders (unless overridden)
  * The _Public_ group represents public / anonymous / not-logged-in users
  * The _Normal users_ group is the default group for logged in users
* The current permissions are shown below
* Click _change_ to alter the current permission level
  * Each higher permission level includes the levels that precede it

Note: global permissions flags (irrespective of folder) can also be defined at the group
level in _Groups_ administration.

Your QIS installation is now complete.

## Troubleshooting

You can usually find an error message (hopefully a useful one) by looking at one of the
various log files.

* If SELinux is enabled, check for any denied operations in `/var/log/audit/audit.log`,
  and see the `audit2allow` tool for a route to fixing these
* If Postgres did not install correctly, check `/var/lib/pgsql/pgstartup.log`
  for a copy of the installation messages
* If Postgres will not start, check `/var/lib/pgsql/data/pg_log/postgresql.log`
  for most startup and runtime errors
* If Apache will not start, check `/var/log/httpd/error_log`
* If Apache is started but you cannot reach the QIS home page (`http://images.example.com/`),
  also check the Apache `error_log`
* If QIS runs in http but not https, also check the Apache `error_log`
* For any other problem, check `/opt/qis/logs/qis.log` and the Apache `error_log`
